@@ -6,6 +6,7 @@ import json
 import os
 import sys
 import shutil
+import secrets
 from datetime import datetime
 from pathlib import Path
 
@@ -36,6 +37,57 @@ def get_template_dir():
     return Path(__file__).parent / "templates"
 
 
+def generate_cuid():
+    """
+    Generate a CUID-like identifier similar to Prisma's @default(cuid()).
+    Format: lowercase alphanumeric string starting with 'c' followed by 24 random characters.
+    """
+    # CUID format: c + timestamp (8 chars) + counter (4 chars) + fingerprint (4 chars) + random (8 chars)
+    # Simplified version: c + 24 random alphanumeric chars
+    chars = 'abcdefghijklmnopqrstuvwxyz0123456789'
+    random_part = ''.join(secrets.choice(chars) for _ in range(24))
+    return f'c{random_part}'
+
+
+def get_active_organization():
+    """
+    Extract the user's active organization from the JWT token.
+    Returns the organization_id or None if not available.
+    """
+    try:
+        # Read the token file
+        if not TOKEN_FILE.exists():
+            return None
+
+        with open(TOKEN_FILE) as f:
+            data = json.load(f)
+
+        access_token = data.get("access_token")
+        if not access_token:
+            return None
+
+        # Decode JWT payload (without verification - just reading the claims)
+        parts = access_token.split(".")
+        if len(parts) < 2:
+            return None
+
+        # Add base64 padding if needed
+        payload = parts[1]
+        payload += "=" * (-len(payload) % 4)
+
+        # Decode the payload
+        decoded = base64.urlsafe_b64decode(payload.encode("utf-8"))
+        token_data = json.loads(decoded.decode("utf-8"))
+
+        # Extract organization_id from JWT claims
+        org_id = token_data.get("organization_id")
+        return org_id
+
+    except Exception as e:
+        click.echo(f"Warning: Could not extract organization from token: {e}")
+        return None
+
+
 def _not_logged_in_exit(open_browser: bool = True) -> None:
     """
     Standardized behavior for commands that require auth:
@@ -139,10 +191,24 @@ def logout():
 @cli.command()
 def start():
     """Initialize a new project with template files."""
+    # Check authentication first to fetch organization
+    _ensure_auth_or_launch_login()
+
     project_dir = Path("./project")
     dataset_dir = Path("./dataset")
     template_dir = get_template_dir()
 
+    # Generate new IDs for project and dataset
+    project_id = generate_cuid()
+    dataset_id = generate_cuid()
+
+    # Fetch active organization
+    organization_id = get_active_organization()
+    if not organization_id:
+        click.echo("\033[91mError: Could not fetch active organization.\033[0m")
+        click.echo("Please ensure you're logged in and have an active organization.")
+        sys.exit(1)
+
     # Create directories
     project_dir.mkdir(exist_ok=True)
     dataset_dir.mkdir(exist_ok=True)
@@ -207,13 +273,32 @@ def start():
         click.echo(f"Failed: {', '.join(failed_files)}")
         return
 
-    click.echo(f"Project files: ./project/")
+    # Update config.json with generated IDs and organization
+    config_file = project_dir / "config.json"
+    try:
+        with open(config_file, "r") as f:
+            config = json.load(f)
+
+        # Update with generated values
+        config["project_id"] = project_id
+        config["dataset_id"] = dataset_id
+        config["organization_id"] = organization_id
+
+        with open(config_file, "w") as f:
+            json.dump(config, f, indent=2)
+
+        click.echo(f"\n\033[1m✓ Configuration updated:\033[0m")
+        click.echo(f"  Project ID: {project_id}")
+        click.echo(f"  Dataset ID: {dataset_id}")
+        click.echo(f"  Organization ID: {organization_id}")
+    except Exception as e:
+        click.echo(f"\n\033[91mWarning: Could not update config.json: {e}\033[0m")
+
+    click.echo(f"\nProject files: ./project/")
     click.echo(f"Dataset files: ./dataset/")
     click.echo(f"\nNext steps:")
-    click.echo(f"  1. Edit config.json and set your organization_id")
-    click.echo(f"  2. Edit files in ./project/ and ./dataset/")
-    click.echo(f"  3. Run \033[1mreinforceenow run\033[0m")
-    click.echo(f"     (New project/dataset will be created automatically on first run)")
+    click.echo(f"  1. Edit files in ./project/ and ./dataset/")
+    click.echo(f"  2. Run \033[1mreinforceenow run\033[0m to start training")
 
 
 def _ensure_auth_or_launch_login() -> None:
@@ -362,6 +447,7 @@ def run(project_dir, dataset_dir):
 
     # Prepare multipart form data for upload
     files = []
+    file_handles = []  # Track file handles for cleanup
     data = {
         "project_id": project_id,
         "dataset_id": dataset_id,
@@ -374,7 +460,9 @@ def run(project_dir, dataset_dir):
     for fname in project_files:
         file_path = project_path / fname
         if file_path.exists():
-            files.append((fname.replace(".", "_"), (fname, open(file_path, "rb"))))
+            fh = open(file_path, "rb")
+            file_handles.append(fh)
+            files.append((fname.replace(".", "_"), (fname, fh, "application/octet-stream")))
         else:
             click.echo(f"Warning: Project file not found: {file_path}")
 
@@ -383,7 +471,9 @@ def run(project_dir, dataset_dir):
     for fname in dataset_files:
         file_path = dataset_path / fname
         if file_path.exists():
-            files.append((fname.replace(".", "_"), (fname, open(file_path, "rb"))))
+            fh = open(file_path, "rb")
+            file_handles.append(fh)
+            files.append((fname.replace(".", "_"), (fname, fh, "application/octet-stream")))
         elif fname == "train.jsonl":
             click.echo(f"Error: Required file not found: {file_path}")
             sys.exit(1)
@@ -400,8 +490,8 @@ def run(project_dir, dataset_dir):
         )
 
         # Close file handles
-        for _, (_, file_obj) in files:
-            file_obj.close()
+        for fh in file_handles:
+            fh.close()
 
         if upload_response.status_code != 200:
             click.echo(f"Upload failed: {upload_response.status_code}")
@@ -419,6 +509,12 @@ def run(project_dir, dataset_dir):
         click.echo("\n\033[1m✓ Training submitted\033[0m\n")
 
     except requests.RequestException as e:
+        # Clean up file handles
+        for fh in file_handles:
+            try:
+                fh.close()
+            except:
+                pass
         click.echo(f"Error: {e}")
         sys.exit(1)
 
